@@ -5,7 +5,7 @@
 # Copyright 1998-1999 - Matt Gerassimoff
 # and Ken Cox <kenstir@senteinc.com>
 #
-# $Id: gnatsweb.pl,v 2.3 1999/05/08 21:51:45 kenstir Exp $
+# $Id: gnatsweb.pl,v 2.4 1999/07/27 16:03:23 kenstir Exp $
 #
 
 #-----------------------------------------------------------------------------
@@ -84,10 +84,10 @@ use CGI qw/:standard :netscape/;
 use CGI::Carp qw/fatalsToBrowser/;
 use Socket;
 use IO::Handle;
-use POSIX;
 
 # get RCS tag as just a number
-( $VERSION = '$Revision: 2.3 $ ' ) =~ s/.Revision: (.*) ../$1/;
+$VERSION = '2.4';
+( $REVISION = '$Revision: 2.4 $ ' ) =~ s/.Revision: (.*) ../$1/;
 
 # width of text fields
 $textwidth = 60;
@@ -237,13 +237,13 @@ sub get_reply
   elsif($state == $CODE_INVALID_CATEGORY)
   {
     $text =~ / (.*)/;
-    gerror("no such category $1");
+    gerror($1);
     client_exit();
   }
   elsif($state == $CODE_INVALID_SUBMITTER)
   {
     $text =~ / (.*)/;
-    gerror("no such submitter $1");
+    gerror($1);
     client_exit();
   }
   elsif($state == $CODE_INVALID_STATE)
@@ -255,13 +255,13 @@ sub get_reply
   elsif($state == $CODE_INVALID_RESPONSIBLE)
   {
     $text =~ / (.*)/;
-    gerror("no such responsible named $1");
+    gerror($1);
     client_exit();
   }
   elsif($state == $CODE_INVALID_DATE)
   {
     $text =~ / (.*)/;
-    gerror("cannot parse the date: $1.");
+    gerror($1);
     client_exit();
   }
   elsif($state == $CODE_UNREADABLE_PR)
@@ -367,7 +367,7 @@ sub sendpr
         $q->textfield(-name=>'email',
                       -default=>$email,
                       -size=>$textwidth),
-	"<tr><td><b>CC:</b><td>",
+	"<tr><td><b>Send a CC of this report only to:</b><td>",
 	$q->textfield(-name=>'cc',
 	              -default=>$cc,
 		      -size=>$textwidth),
@@ -400,10 +400,13 @@ sub sendpr
     }
     elsif ($fieldnames{$_} & $MULTILINE)
     {
+      my $rows = 4;
+      $rows = 8 if /Description/;
+      $rows = 2 if /Environment/;
       print "<tr><td valign=top><b>$_:</b><td>",
             $q->textarea(-name=>$_,
                          -cols=>$textwidth,
-                         -rows=>4,
+                         -rows=>$rows,
                          -default=>$default);
     }
     else
@@ -436,7 +439,8 @@ sub validate_email_field
   {
     return "$fieldname is blank";
   }
-  if (!$blank && $fieldval !~ /^\s*\S+\@\S+\.\S+\s*$/)
+  $email_addr = '\S+(\@\S+\.\S+)?';
+  if (!$blank && $fieldval !~ /^\s*($email_addr\s*)*$/)
   {
     return "'$fieldval' doesn't look like a valid email address (xxx\@xxx.xxx)";
   }
@@ -511,7 +515,7 @@ CC: $fields{'cc'}
 Subject: $fields{'Synopsis'}
 From: $fields{'email'}
 Reply-To: $fields{'email'}
-X-gnatsweb-version: $VERSION
+X-Send-Pr-Version: gnatsweb-$VERSION ($REVISION)
 X-GNATS-Notify: $fields{'X-GNATS-Notify'}
 
 EOT
@@ -581,10 +585,11 @@ sub view
   print $q->submit('cmd', 'view audit-trail') if (!$viewaudit);
 
   # print a convenient 'mailto' link
-  my $mailto  = interested_parties($pr, %fields);
-  my $subject = "Re: $fields{'Category'}/$pr";
+  my $mailto  = $q->escape(scalar(interested_parties($pr, %fields)));
+  my $subject = $q->escape("Re: $fields{'Category'}/$pr");
+  my $body    = $q->escape(get_viewpr_url($pr));
   print " or ",
-        "<a href=\"mailto:$mailto?Subject=$subject\">",
+        "<a href=\"mailto:$mailto?Subject=$subject&Body=$body\">",
         "send email to interested parties</a>";
 
   print $q->hr(),
@@ -705,10 +710,13 @@ sub edit
     }
     elsif ($fieldnames{$_} && $fieldnames{$_} & $MULTILINE)
     {
+      my $rows = 4;
+      $rows = 8 if /Description/;
+      $rows = 2 if /Environment/;
       print "<tr><td valign=top><b>$_:</b><td>",
             $q->textarea(-name=>$_,
                          -cols=>$textwidth,
-                         -rows=>4,
+                         -rows=>$rows,
                          -default=>$fields{$_});
     }
     else
@@ -744,6 +752,12 @@ sub edit
   page_end_html($page);
 }
 
+sub get_viewpr_url
+{
+  my $pr = shift;
+  $q->url() . "?cmd=view&database=$database&pr=$pr";
+}
+
 sub submitedit
 {
   my $page = 'Edit PR Results';
@@ -874,7 +888,10 @@ sub submitedit
 	    }
             else
             {
-              $audittrail .= "$_-Changed-Why:\n" . $fields{"$_-Why"};
+              # Indent the "Why" value.
+              my $tmp = $fields{"$_-Why"};
+              $tmp =~ s/^/    /gm;
+              $audittrail .= "$_-Changed-Why:\n" . $tmp;
             }
 	  }
 	}
@@ -913,10 +930,12 @@ sub submitedit
       #last LOCKED; # debug
       if($mailto ne "" && $audittrail ne "")
       {
+        # Use email address in responsible file as From, if present.
+        my $from = $responsible_address{$person} || $user;
         if(open(MAILER, "|$site_mailer"))
 	{
 	  print MAILER "To: $mailto\n";
-	  print MAILER "From: $user\n";
+	  print MAILER "From: $from\n";
 	  print MAILER "Subject: Re: $fields{'Category'}/$pr\n\n";
           if ($oldfields{'Synopsis'} eq $fields{'Synopsis'})
           {
@@ -929,7 +948,7 @@ sub submitedit
           }
           print MAILER "$audittrail\n";
           # Print URL so that HTML-enabled mail readers can jump to the PR.
-          print MAILER $q->url() . "?cmd=view&database=$database&pr=$fields{'Number'}\n";
+          print MAILER get_viewpr_url($pr), "\n";
           if(!close(MAILER))
 	  {
             page_heading($page, 'Error');
@@ -1338,6 +1357,8 @@ sub display_query_results
     {
       $q->param(-name=>'sortby', -value=>$ufield);
       my $href = $q->self_url();
+      # 6/25/99 kenstir: CEL claims this avoids a problem w/ apache+mod_perl.
+      $href =~ s/^[^?]*\?/$sn\?/; #CEL
       print "<th><a href=\"$href\">$ufield</a>";
     }
     else
@@ -1748,7 +1769,8 @@ sub page_heading
     $rightcol = '&nbsp;';
   }
 
-  print $q->table($q->Tr($q->td({-nowrap=>1}, font({-size=>'+3'}, $leftcol)),
+  print $q->table($q->Tr($q->td({-nowrap=>1}, $q->font({-size=>'+3'},
+                                                       $leftcol)),
                          # this column serves as empty expandable filler
                          $q->td({-width=>'100%'}, '&nbsp;'),
                          $q->td({-nowrap=>1}, $rightcol)));
